%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     This code analyze the performance of algorithms in terms of MSE
%
%          and call the OnlineRelabeling function to deal 
%           with the label switching problem
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Matlab Codes associated to the paper
%       ``A Bayesian Perspective on Multiple Source
%        Localization in Wireless Sensor Network'' 
% by T.L.T Nguyen, F. Septier, H. Rajaona, G.W. Peters, I. Nevat,
%       Y. Delignon
%  
%       Date: Sep. 2015
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all;


% Define the output of algorithm to load (TO BE CHANGED ACCORDING TO THE
% SCENARIO !!)
DataFileSMC=['SavedData/Results_SMC_TargetScenario_NbSensors=100_QuantNbLevel=8_NoiseVar=1_NumMcRun=_Model1_PercentageCESS=0.9_NbPart=5_NbMCMCMove=5_NbBlock=1_AdaptiveTemperature'];
DataFileIS=['SavedData/Results_IS_TargetScenario_NbSensors=100_QuantNbLevel=8_NoiseVar=1_NumMcRun=_Model1_NbPartTotal=9000.mat']

% Load Results from the SMC
eval(['load ',DataFileSMC])


for n=1:NbMCruns
    
    ParticlesBeta=ResultsSMC.SavedParticles.Beta(:,:,n);
    ParticlesLogPower=ResultsSMC.SavedParticles.LogPower(:,:,n);
    Weights=ResultsSMC.SavedParticles.NormalizedWeights(:,:,n);
    [RelabeledParticlesBeta,RelabeledParticlesLogPower,NormWeights]=OnlineRelabelingv1(ParticlesBeta,ParticlesLogPower,Weights,0);

    BetaPosteriorMean(:,n)=sum(RelabeledParticlesBeta.*repmat(NormWeights,size(RelabeledParticlesBeta,1),1),2);
    LogPowerPosteriorMean(:,n)=sum(RelabeledParticlesLogPower.*repmat(NormWeights,size(RelabeledParticlesLogPower,1),1),2);
    LogPowerPosteriorMean2(:,n)=sum(exp(RelabeledParticlesLogPower).*repmat(NormWeights,size(RelabeledParticlesLogPower,1),1),2);

    % Find the minimum Associate MSE
    AllPermutations=perms([1:ModelToAnalyze]);
    TempMSE=zeros(size(AllPermutations,1),1);
    for k=1:size(AllPermutations,1)
        IndexTargetPos=reshape(repmat((AllPermutations(k,:)-1)*2,2,1)+repmat([1 2]',1,ModelToAnalyze),1,2*ModelToAnalyze);
        TempMSE(k)=sum(abs(TrueTargetPosition(1:2*Target.Nb,n)-BetaPosteriorMean(IndexTargetPos,n)).^2);
    end

    %figure(4);clf;plot(TempMSE)
    [C,I]=min(TempMSE);
    IndexTargetPos=reshape(repmat((AllPermutations(I(1),:)-1)*2,2,1)+repmat([1 2]',1,ModelToAnalyze),1,2*ModelToAnalyze);

    BetaPosteriorMean(:,n)=BetaPosteriorMean(IndexTargetPos,n);
    LogPowerPosteriorMean(:,n)=LogPowerPosteriorMean(AllPermutations(I(1),:),n);
    LogPowerPosteriorMean2(:,n)=LogPowerPosteriorMean2(AllPermutations(I(1),:),n);

    % Saved the MSE
    SavedMSE.MSE_perrun(1,n)=C;
    SavedMSE.MSE_Power_perrun(1,n)=sum(abs(TrueTargetPower(:,n)-exp(LogPowerPosteriorMean(:,n))).^2);
    SavedMSE.MSE_Power2_perrun(1,n)=sum(abs(TrueTargetPower(:,n)-LogPowerPosteriorMean2(:,n)).^2);       
    
    
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%       For the Importance Sampler
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Load Results from the IS
eval(['load ',DataFileIS])

 for n=1:NbMCruns
    
    ParticlesBeta=ResultsSMC.SavedParticles.Beta(:,:,n);
    ParticlesLogPower=ResultsSMC.SavedParticles.LogPower(:,:,n);
    Weights=ResultsSMC.SavedParticles.NormalizedWeights(:,:,n);

    % Threshold to eliminate particles with low weights -
    % otherwise to complex to compute the relabeling !!
    Threshold=1e-5;
    IndexSup=find(Weights>Threshold);
    NewWeights=Weights(IndexSup);
    NewWeights=NewWeights/sum(NewWeights);
    NewParticlesBeta=ParticlesBeta(:,IndexSup);
    NewParticlesLogPower=ParticlesLogPower(:,IndexSup);                

    [RelabeledParticlesBeta,RelabeledParticlesLogPower,NormWeights]=OnlineRelabelingv1(NewParticlesBeta,NewParticlesLogPower,NewWeights,0);

    

    BetaPosteriorMean(:,n)=sum(RelabeledParticlesBeta.*repmat(NormWeights,size(RelabeledParticlesBeta,1),1),2);
    LogPowerPosteriorMean(:,n)=sum(RelabeledParticlesLogPower.*repmat(NormWeights,size(RelabeledParticlesLogPower,1),1),2);
    LogPowerPosteriorMean2(:,n)=sum(exp(RelabeledParticlesLogPower).*repmat(NormWeights,size(RelabeledParticlesLogPower,1),1),2);

    % Find the minimum Associate MSE
    AllPermutations=perms([1:ModelToAnalyze]);
    TempMSE=zeros(size(AllPermutations,1),1);
    for k=1:size(AllPermutations,1)
        IndexTargetPos=reshape(repmat((AllPermutations(k,:)-1)*2,2,1)+repmat([1 2]',1,ModelToAnalyze),1,2*ModelToAnalyze);
        TempMSE(k)=sum(abs(TrueTargetPosition(1:2*Target.Nb,n)-BetaPosteriorMean(IndexTargetPos,n)).^2);
    end

    
    [C,I]=min(TempMSE);
    IndexTargetPos=reshape(repmat((AllPermutations(I(1),:)-1)*2,2,1)+repmat([1 2]',1,ModelToAnalyze),1,2*ModelToAnalyze);

    BetaPosteriorMean(:,n)=BetaPosteriorMean(IndexTargetPos,n);
    LogPowerPosteriorMean(:,n)=LogPowerPosteriorMean(AllPermutations(I(1),:),n);
    LogPowerPosteriorMean2(:,n)=LogPowerPosteriorMean2(AllPermutations(I(1),:),n);                   
    
    
    % Save the MSE for this realization of observations
    SavedMSE.MSE_perrunIS(1,n)=C;
    SavedMSE.MSE_Power_perrunIS(1,n)=sum(abs(TrueTargetPower(1:Target.Nb,n)-exp(LogPowerPosteriorMean(:,n))).^2);
    SavedMSE.MSE_Power2_perrunIS(1,n)=sum(abs(TrueTargetPower(1:Target.Nb,n)-LogPowerPosteriorMean2(:,n)).^2);                    
    
    

end





