%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     This code computes the log likelihood the different parameters 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Matlab Codes associated to the paper
%       ``A Bayesian Perspective on Multiple Source
%        Localization in Wireless Sensor Network'' 
% by T.L.T Nguyen, F. Septier, H. Rajaona, G.W. Peters, I. Nevat,
%       Y. Delignon
%  
%       Date: Sep. 2015
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [LogLikelihood]=ComputationLogLikelihood(Sensors,TargetPosition,Target,TargetPower)





% ----- CODE WITH IMPERFECT CHANNEL TO FUSION CENTER ---
ThresholdLike=[-1e50 Sensors.Threshold 1e50];

 for nbTarget=1:Target.Nb
    IndexTarget=(nbTarget-1)*Target.Dim/Target.Nb+1:(nbTarget)*Target.Dim/Target.Nb;
    DistanceGrid(nbTarget,:)=sqrt(sum(abs(repmat(TargetPosition(IndexTarget,1),1,Sensors.Nb)-Sensors.Position).^2,1));
    A(nbTarget,:)=sqrt(TargetPower(nbTarget)*(Target.d0./DistanceGrid(nbTarget,:)).^Sensors.SignalDecay);  
end
A=repmat(sum(A,1),Sensors.NbLevel,1);  



ProbaTransition=Sensors.TransMatrix(:,Sensors.ReceivedSymbols+1);
AllPossibleQuantLevel=repmat([0:Sensors.NbLevel-1]',1,Sensors.Nb);
ProbaTest=normcdf((ThresholdLike(AllPossibleQuantLevel+2)-A)/sqrt(Sensors.NoiseVariance))-normcdf((ThresholdLike(AllPossibleQuantLevel+1)-A)/sqrt(Sensors.NoiseVariance));

LogLikelihood=sum(log(sum(ProbaTransition.*ProbaTest,1)+1e-323));
